use rand::{seq::SliceRandom as _, Rng as _};

pub fn main() {
    let passphrase = create_passphrase(" ", 8);
    println!("{passphrase}");
    let pin = create_pin(8);
    println!("{pin}");
    println!("Press Enter");
    let mut input = String::new();
    std::io::stdin().read_line(&mut input).ok();
}

pub fn create_passphrase(separator: &str, len: usize) -> String {
    let dice = Dice::default();
    let words: Vec<_> = (0..len).map(|_| dice.pick_word()).collect();
    words.join(separator)
}

/// Generates a decimal PIN
///
/// PINs are returned as a String because you do not do math with PINs, therefore they are not numbers.
pub fn create_pin(len: usize) -> String {
    let digits: Vec<_> = (0..len)
        .map(|_| format!("{}", rand::thread_rng().gen_range(0..=9)))
        .collect();
    digits.join("")
}

pub struct Dice {
    words: Vec<String>,
}

impl Default for Dice {
    fn default() -> Self {
        let wordlist = include_str!("eff_short_wordlist_1.txt");
        let words: Vec<_> = wordlist.split('\n').map(|s| s.to_string()).collect();

        assert_eq!(words.len(), 1253);
        assert_eq!(words[0], "acid");
        assert_eq!(words[600], "large");
        assert_eq!(words[1252], "zoom");

        Self { words }
    }
}

impl Dice {
    pub fn pick_word(&self) -> &str {
        self.words
            .choose(&mut rand::thread_rng())
            .expect("`choose` should always return `Some`")
    }
}
